@@ -729,18 +729,18 @@ _g_mount_guess_content_type (GMount              *mount,
 		g_ptr_array_add (types, g_strdup ("x-content/image-picturecd"));
 	}
 
-	if (_check_file (root, ".autorun", TRUE) ||
-	    _check_file (root, "autorun", TRUE) ||
-	    _check_file (root, "autorun.sh", TRUE) ||
-	    _check_file_case_insensitive (root, "autorun.exe", TRUE) ||
-	    _check_file_case_insensitive (root, "autorun.inf", FALSE)) {
+	if (g_file_is_native (root) &&
+	    (_check_file (root, ".autorun", TRUE) ||
+	     _check_file (root, "autorun", TRUE) ||
+	     _check_file (root, "autorun.sh", TRUE) ||
+	     _check_file_case_insensitive (root, "autorun.exe", TRUE) ||
+	     _check_file_case_insensitive (root, "autorun.inf", FALSE))) {
 		/* http://standards.freedesktop.org/autostart-spec/autostart-spec-latest.html */
-
+		
 		/* http://bugzilla.gnome.org/show_bug.cgi?id=509823#c3 for the autorun.exe and autorun.inf stuff */
 		g_ptr_array_add (types, g_strdup ("x-content/software"));
 	}
 
-
 no_sniff:
 	
 	g_ptr_array_add (types, NULL);
@@ -1269,20 +1269,80 @@ nautilus_autorun (GMount *mount, NautilusAutorunOpenWindow open_window_func, gpo
 					   data);
 }
 
-char **
-nautilus_autorun_get_x_content_types_for_mount (GMount      *mount,
-						gboolean     force_rescan)
+typedef struct {
+	NautilusAutorunGetContent callback;
+	gpointer user_data;
+} GetContentTypesData;
+
+static void
+get_types_cb (GObject *source_object,
+	      GAsyncResult *res,
+	      gpointer user_data)
 {
+	GetContentTypesData *data;
+	char **types;
+
+	data = user_data;
+	types = _g_mount_guess_content_type_finish (G_MOUNT (source_object), res, NULL);
+
+	if (data->callback) {
+		data->callback (types, data->user_data);
+	}
+	g_strfreev (types);
+	g_free (data);
+}
+
+void
+nautilus_autorun_get_x_content_types_for_mount_async (GMount *mount,
+						      NautilusAutorunGetContent callback,
+						      GCancellable *cancellable,
+						      gpointer user_data)
+{
+	char **cached;
+	GetContentTypesData *data;
+	
+	if (mount == NULL) {
+		if (callback) {
+			callback (NULL, user_data);
+		}
+		return;
+	}
+
+	cached = g_object_get_data (G_OBJECT (mount), "content-type-cache");
+	if (cached != NULL) {
+		if (callback) {
+			callback (cached, user_data);
+		}
+		return;
+	}
+
+	data = g_new (GetContentTypesData, 1);
+	data->callback = callback;
+	data->user_data = user_data;
+	
+	_g_mount_guess_content_type_async (mount,
+					   FALSE,
+					   cancellable,
+					   get_types_cb,
+					   data);
+}
+
+
+char **
+nautilus_autorun_get_cached_x_content_types_for_mount (GMount      *mount)
+{
+	char **cached;
+	
 	if (mount == NULL) {
 		return NULL;
 	}
 
-	/* since we always guess the content type at mount type, we're guaranteed
-	 * to get the cached results
-	 *
-	 * TODO: Really? what if we didn't mount the mount ourself?
-	 */
-	return _g_mount_guess_content_type (mount, force_rescan, NULL);
+	cached = g_object_get_data (G_OBJECT (mount), "content-type-cache");
+	if (cached != NULL) {
+		return g_strdupv (cached);
+	}
+
+	return NULL;
 }
 
 
