@@ -30,7 +30,7 @@
 #include "nautilus-dnd.h"
 #include "nautilus-directory.h"
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-string.h>
+#include <string.h>
 
 static gboolean
 nautilus_drag_can_accept_files (NautilusFile *drop_target_item)
@@ -111,10 +111,13 @@ nautilus_drag_can_accept_info (NautilusFile *drop_target_item,
 			return nautilus_drag_can_accept_items (drop_target_item, items);
 
 		case NAUTILUS_ICON_DND_URI_LIST:
-		case NAUTILUS_ICON_DND_URL:
+		case NAUTILUS_ICON_DND_NETSCAPE_URL:
 		case NAUTILUS_ICON_DND_TEXT:
 			return nautilus_drag_can_accept_files (drop_target_item);
 
+		case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
+			return nautilus_drag_can_accept_files (drop_target_item); /* Check if we can accept files at this location */
+
 		case NAUTILUS_ICON_DND_KEYWORD:
 			return TRUE;
 
@@ -131,11 +134,11 @@ nautilus_drag_can_accept_info (NautilusFile *drop_target_item,
 			g_assert_not_reached ();
 			return FALSE;
 	}
-	
 }
 
 void
-nautilus_drag_file_receive_dropped_keyword (NautilusFile *file, const char *keyword)
+nautilus_drag_file_receive_dropped_keyword (NautilusFile *file,
+					    const char *keyword)
 {
 	GList *keywords, *word;
 
@@ -156,7 +159,7 @@ nautilus_drag_file_receive_dropped_keyword (NautilusFile *file, const char *keyw
 			g_list_free_1 (word);
 		}
 	}
-	
+
 	nautilus_file_set_keywords (file, keywords);
 	eel_g_list_free_deep (keywords);
 }
