@@ -973,7 +973,9 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 					       mount_not_mounted_callback, data);
 		g_object_unref (location);
 		g_object_unref (mount_op);
-		
+
+		nautilus_file_unref (file);
+
 		return;
 	}
 	
@@ -1295,7 +1297,7 @@ add_extension_extra_widgets (NautilusWindow *window, GFile *location)
 }
 
 static void
-nautilus_window_show_x_content_bar (NautilusWindow *window, GMount *mount, char **x_content_types)
+nautilus_window_show_x_content_bar (NautilusWindow *window, GMount *mount, const char **x_content_types)
 {
 	unsigned int n;
 
@@ -1334,6 +1336,70 @@ nautilus_window_show_trash_bar (NautilusWindow *window)
 	nautilus_window_add_extra_location_widget (window, bar);
 }
 
+typedef struct {
+	NautilusWindow *window;
+	GCancellable *cancellable;
+	GMount *mount;
+} FindMountData;
+
+static void
+found_content_type_cb (const char **x_content_types, FindMountData *data)
+{
+	NautilusWindow *window;
+	
+	if (g_cancellable_is_cancelled (data->cancellable)) {
+		goto out;
+	}
+	
+	window = data->window;
+	
+	if (x_content_types != NULL && x_content_types[0] != NULL) {
+		nautilus_window_show_x_content_bar (window, data->mount, x_content_types);
+		update_extra_location_widgets_visibility (window);
+	}
+
+	window->details->find_mount_cancellable = NULL;
+
+ out:
+	g_object_unref (data->mount);
+	g_object_unref (data->cancellable);
+	g_free (data);
+}
+
+static void
+found_mount_cb (GObject *source_object,
+		GAsyncResult *res,
+		gpointer user_data)
+{
+	FindMountData *data = user_data;
+	GMount *mount;
+	NautilusWindow *window;	
+
+	if (g_cancellable_is_cancelled (data->cancellable)) {
+		goto out;
+	}
+	
+	window = data->window;
+	
+	mount = g_file_find_enclosing_mount_finish (G_FILE (source_object),
+						    res,
+						    NULL);
+	if (mount != NULL) {
+		data->mount = mount;
+		nautilus_autorun_get_x_content_types_for_mount_async (mount,
+								      (NautilusAutorunGetContent)found_content_type_cb,
+								      data->cancellable,
+								      data);
+		return;
+	}
+	
+	window->details->find_mount_cancellable = NULL;
+
+ out:
+	g_object_unref (data->cancellable);
+	g_free (data);
+}
+
 /* Handle the changes for the NautilusWindow itself. */
 static void
 update_for_new_location (NautilusWindow *window)
@@ -1343,8 +1409,7 @@ update_for_new_location (NautilusWindow *window)
 	NautilusDirectory *directory;
 	gboolean location_really_changed;
 	char *uri;
-	char **x_content_types;
-	GMount *mount;
+	FindMountData *data;
         
         new_location = window->details->pending_location;
         window->details->pending_location = NULL;
@@ -1403,15 +1468,23 @@ update_for_new_location (NautilusWindow *window)
 			nautilus_window_show_trash_bar (window);
 		}
 
-		mount = nautilus_file_get_mount (file);
-		if (mount != NULL) {
-			x_content_types = nautilus_autorun_get_x_content_types_for_mount (mount, FALSE);
-			if (x_content_types != NULL && x_content_types[0] != NULL) {
-				nautilus_window_show_x_content_bar (window, mount, x_content_types);
-			}
-			g_strfreev (x_content_types);
-			g_object_unref (mount);
+		/* need the mount to determine if we should put up the x-content cluebar */
+		if (window->details->find_mount_cancellable != NULL) {
+			g_cancellable_cancel (window->details->find_mount_cancellable);
+			window->details->find_mount_cancellable = NULL;
 		}
+		
+		data = g_new (FindMountData, 1);
+		data->window = window;
+		data->cancellable = g_cancellable_new ();
+		data->mount = NULL;
+		
+		window->details->find_mount_cancellable = data->cancellable;
+		g_file_find_enclosing_mount_async (window->details->location, 
+						   G_PRIORITY_DEFAULT, 
+						   data->cancellable,
+						   found_mount_cb,
+						   data);
 
 		nautilus_directory_unref (directory);
 
